;;;---------------------------------------------------------------;;;
;;;                                                               ;;;
;;;                  HIP ROOF - 2D RIDGE EDGES                    ;;;
;;;        (SIMPLE VERSION - MAY GIVE NOT CORRECT RESULT)         ;;;
;;;         (SLOWER BUT CORRECT VERSION - TRY *-NEW.LSP)          ;;;
;;;                                                               ;;;
;;;  Author : Marko Ribar, d.i.a. (architect)                     ;;;
;;;  Autolisp routine for AutoCAD or BricsCAD                     ;;;
;;;  Copyright (C) 2021.                                          ;;;
;;;  Tested and programmed on Windows 10 OS                       ;;;
;;;  Bugs or failures may appear and aren't authors responsibility;;;
;;;  LSP file is free to be used for improvements...              ;;;
;;;  All relevant suggestions are welcomed...                     ;;;
;;;                                                               ;;;
;;;---------------------------------------------------------------;;;

(defun c:roof-command ( / *error* acos angle3d unique unit v^v clean_poly clockwise-lw cmde ucsf s ti lw lwx pl tl v1 v2 tll tlll ss 3df regobj 3ds e lil lix ch chh slope ang var )

  (vl-load-com)

  (defun *error* ( m )
    (if cmde
      (setvar 'cmdecho cmde)
    )
    (if m
      (prompt m)
    )
    (princ)
  )

  (defun acos ( x )
    (cond
      ( (equal x 1.0 1e-8) 0.0 )
      ( (equal x -1.0 1e-8) pi )
      ( (and (not (minusp x)) (equal x 0.0 1e-8)) (/ pi 2.0) )
      ( (and (minusp x) (equal x 0.0 1e-8)) (* 3.0 (/ pi 2.0)) )
      ( (atan (sqrt (- 1.0 (* x x))) x) )
    )
  )

  (defun angle3d ( p1 por p2 / vec1 vec2 dd ang )
    (setq vec1 (unit (mapcar '- p1 por))
          vec2 (unit (mapcar '- p2 por))
          dd (distance vec1 vec2)
          ang (acos (- 1.0 (/ (expt dd 2) 2.0)))
    )
    (if (minusp ang) (+ ang pi) ang)
  )

  (defun unique ( l / a ll )
    (while (setq a (car l))
      (if (vl-some (function (lambda ( x ) (equal x a 1e-6))) (cdr l))
        (setq ll (cons a ll) l (vl-remove-if (function (lambda ( x ) (equal x a 1e-6))) (cdr l)))
        (setq ll (cons a ll) l (cdr l))
      )
    )
    (reverse ll)
  )

  (defun unit ( v / d )
    (if (not (equal (setq d (distance v '(0.0 0.0 0.0))) 0.0 1e-8))
      (mapcar (function (lambda ( x ) (/ x d))) v)
    )
  )

  (defun v^v ( u v )
    (list
      (- (* (cadr u) (caddr v)) (* (caddr u) (cadr v)))
      (- (* (caddr u) (car v)) (* (car u) (caddr v)))
      (- (* (car u) (cadr v)) (* (cadr u) (car v)))
    )
  )

  (defun clean_poly ( lw / pll pos pre suf k lwx )
    (setq pll (reverse (cdr (reverse pl))))
    (setq pos (mapcar (function (lambda ( a b c ) (if (equal (distance a c) (+ (distance a b) (distance b c)) 1e-6) t nil))) (cons (last pll) pll) pll (append (cdr pll) (list (car pll)))))
    (if (apply (function or) pos)
      (progn
        (setq k -1)
        (setq pos (vl-remove nil (mapcar (function (lambda ( x ) (progn (setq k (1+ k)) (if (eq x t) k)))) pos)))
        (foreach p pos
          (setq pos (subst (nth p pll) p pos))
        )
        (foreach p pos
          (setq pll (vl-remove-if (function (lambda ( x ) (equal x p 1e-8))) pll))
        )
        (setq pre (reverse (cdr (member (assoc 10 (setq lwx (entget lw))) (reverse lwx)))))
        (setq pre (subst (cons 90 (length pll)) (assoc 90 pre) pre))
        (setq suf (append (mapcar (function (lambda ( x ) (cons 10 (trans x 0 (cdr (assoc 210 lwx)))))) pll) (list (assoc 210 lwx))))
        (entmod (append pre suf))
      )
    )
    (entupd lw)
  )

  (defun clockwise-lw ( lw / minpt maxpt p1 p2 p3 p4 pmax )
    (vla-getboundingbox (vlax-ename->vla-object lw) 'minpt 'maxpt)
    (mapcar (function set) '(minpt maxpt) (mapcar (function safearray-value) (list minpt maxpt)))
    (setq p1 minpt p2 (list (car maxpt) (cadr minpt)) p3 maxpt p4 (list (car minpt) (cadr maxpt)))
    (setq p1 (vlax-curve-getparamatpoint lw (vlax-curve-getclosestpointto lw p1)))
    (setq p2 (vlax-curve-getparamatpoint lw (vlax-curve-getclosestpointto lw p2)))
    (setq p3 (vlax-curve-getparamatpoint lw (vlax-curve-getclosestpointto lw p3)))
    (setq p4 (vlax-curve-getparamatpoint lw (vlax-curve-getclosestpointto lw p4)))
    (setq pmax (max p1 p2 p3 p4))
    (cond
      ( (and (= pmax p1) (> p2 p4))
        t
      )
      ( (and (= pmax p2) (> p3 p1))
        t
      )
      ( (and (= pmax p3) (> p4 p2))
        t
      )
      ( (and (= pmax p4) (> p1 p3))
        t
      )
      ( t nil )
    )
  )

  (setq cmde (getvar 'cmdecho))
  (setvar 'cmdecho 0)
  (vl-cmdf "_.UNDO" "_BE")
  (if (= 0 (getvar 'worlducs))
    (progn
      (vl-cmdf "_.UCS" "_W")
      (setq ucsf t)
    )
  )
  (prompt "\nPick a closed polygonal LWPOLYLINE...")
  (if (setq s (ssget "_+.:E:S" '((0 . "LWPOLYLINE") (-4 . "&=") (70 . 1) (-4 . "<not") (-4 . "<>") (42 . 0.0) (-4 . "not>"))))
    (progn
      (setq lw (ssname s 0))
      (setq lwx (entget lw))
      (setq pl (mapcar (function (lambda ( p ) (mapcar (function +) (list 0 0) (trans p lw 0)))) (mapcar (function cdr) (vl-remove-if (function (lambda ( x ) (/= (car x) 10))) lwx))))
      (setq pl (append pl (list (car pl))))
      (if (clockwise-lw lw)
        (setq pl (reverse pl))
      )
      (clean_poly lw)
      (setq lwx (entget lw))
      (setq pl (mapcar (function (lambda ( p ) (mapcar (function +) (list 0 0) (trans p lw 0)))) (mapcar (function cdr) (vl-remove-if (function (lambda ( x ) (/= (car x) 10))) lwx))))
      (setq pl (unique pl))
      (setq pl (append pl (list (car pl))))
      (if (clockwise-lw lw)
        (setq pl (reverse pl))
      )
      (setq tl (mapcar (function (lambda ( a b ) (list (append a (list 0.0)) (append b (list 0.0))))) (reverse (cdr (reverse pl))) (cdr pl)))
      (initget 1 "2D 3D")
      (setq ch (getkword "\nChoose option [2D/3D] : "))
      (if (= ch "3D")
        (progn
          (initget 1 "Slope Angle")
          (setq chh (getkword "\nChoose option [Angle/Slope] : "))
          (cond
            ( (= chh "Slope")
              (initget 6)
              (setq slope (getreal "\nEnter slope in percentage <100> : "))
              (if (null slope)
                (setq slope 100.0)
              )
              (setq ang (cvunit (atan (/ slope 100.0)) "radian" "degree"))
            )
            ( t
              (initget 6)
              (setq ang (getreal "\nEnter angle in decimal degrees <45> : "))
              (if (null ang)
                (setq ang 45.0)
              )
            )
          )
        )
        (setq ang 45.0)
      )
      (setq ti (car (_vl-times)))
      (foreach tt tl
        (setq v1 (unit (mapcar (function -) (cadr tt) (car tt))))
        (setq v2 (mapcar (function -) (append (mapcar (function +) (list 0.0 0.0) (polar (car tt) (+ (angle (car tt) (cadr tt)) (* 0.5 pi)) (cos (cvunit ang "degree" "radian")))) (list (sin (cvunit ang "degree" "radian")))) (car tt)))
        (setq tll (cons (list tt (unit (v^v v1 v2))) tll))
      )
      (setq tll (reverse tll))
      (setq tlll (mapcar (function (lambda ( t1 t2 t3 / v1 v2 ip p1 p2 ) (setq v1 (unit (v^v (cadr t1) (cadr t2))) v2 (unit (v^v (cadr t2) (cadr t3))) ip (inters (caar t2) (mapcar (function +) (caar t2) v1) (caar t3) (mapcar (function +) (caar t3) v2) nil)) (if (and (null ip) (not (equal v1 (mapcar (function -) v2) 1e-6)) (not (equal v1 v2 1e-6)) (coplanar-p (caar t2) (mapcar (function +) (caar t2) v1) (caar t3) (mapcar (function +) (caar t3) v2))) (setq ip (trans (inters (append (mapcar (function +) (list 0.0 0.0) (trans (caar t2) 0 (v^v v1 v2))) (list (setq z (caddr (trans (caar t2) 0 (v^v v1 v2)))))) (append (mapcar (function +) (list 0.0 0.0) (trans (mapcar (function +) (caar t2) v1) 0 (v^v v1 v2))) (list z)) (append (mapcar (function +) (list 0.0 0.0) (trans (caar t3) 0 (v^v v1 v2))) (list z)) (append (mapcar (function +) (list 0.0 0.0) (trans (mapcar (function +) (caar t3) v2) 0 (v^v v1 v2))) (list z)) nil) (v^v v1 v2) 0))) (if ip (progn (cond ( (minusp (- (caddr ip) (cdr (assoc 38 lwx)))) (if (minusp (caddr v1)) (setq v1 (mapcar (function -) v1))) (if (minusp (caddr v2)) (setq v2 (mapcar (function -) v2))) ) ( t (if (not (minusp (caddr v1))) (setq v1 (mapcar (function -) v1))) (if (not (minusp (caddr v2))) (setq v2 (mapcar (function -) v2))) ) ) (setq p1 (mapcar (function +) (caar t2) (mapcar (function *) v1 (list 1e+5 1e+5 1e+5)))) (setq p2 (mapcar (function +) (cadar t2) (mapcar (function *) v2 (list 1e+5 1e+5 1e+5)))) (list (car t2) (cadr t2) (list ip p1 p2))) (list (car t2) (cadr t2) (list (caar t2) (if (< (angle3d (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (cadar t2) (caar t2))) (list 0.0 0.0 0.0) (mapcar (function +) (list 0.0 0.0) (if (minusp (caddr v1)) (mapcar (function -) v1) v1))) (* 0.5 pi)) (append (polar (mapcar (function +) (list 0.0 0.0) (caar t2)) (+ (angle (caar t2) (cadar t2)) (* 0.5 pi)) (* 1e+3 (cos (cvunit ang "degree" "radian")))) (list (+ (* 1e+3 (sin (cvunit ang "degree" "radian"))) (cdr (assoc 38 lwx))))) (mapcar (function +) (caar t2) (mapcar (function *) (if (minusp (caddr v1)) (list (- 1e+5) (- 1e+5) (- 1e+5)) (list 1e+5 1e+5 1e+5)) v1))) (if (< (angle3d (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (caar t2) (cadar t2))) (list 0.0 0.0 0.0) (mapcar (function +) (list 0.0 0.0) (if (minusp (caddr v2)) (mapcar (function -) v2) v2))) (* 0.5 pi)) (append (polar (mapcar (function +) (list 0.0 0.0) (cadar t2)) (+ (angle (caar t2) (cadar t2)) (* 0.5 pi)) (* 1e+3 (cos (cvunit ang "degree" "radian")))) (list (+ (* 1e+3 (sin (cvunit ang "degree" "radian"))) (cdr (assoc 38 lwx))))) (mapcar (function +) (cadar t2) (mapcar (function *) (if (minusp (caddr v2)) (list (- 1e+5) (- 1e+5) (- 1e+5)) (list 1e+5 1e+5 1e+5)) v2))) (cadar t2)))))) (cons (last tll) (reverse (cdr (reverse tll)))) tll (append (cdr tll) (list (car tll)))))
      (setq ss (ssadd))
      (foreach tt tlll
        (vl-cmdf "_.UCS" "_ZA" "_non" (caar tt) "_non" (mapcar (function +) (caar tt) (cadr tt)))
        (if (= (length (last tt)) 3)
          (if (minusp (caddr (car (last tt))))
            (cond
              ( (and (>= (angle3d (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (cadar tt) (caar tt))) (list 0.0 0.0 0.0) (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (caar tt) (car (last tt))))) (* 0.5 pi)) (>= (angle3d (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (caar tt) (cadar tt))) (list 0.0 0.0 0.0) (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (cadar tt) (car (last tt))))) (* 0.5 pi)))
                (vl-cmdf "_.PLINE" "_non" (trans (cadr (last tt)) 0 1) "_non" (trans (caar tt) 0 1) "_non" (trans (cadar tt) 0 1) "_non" (trans (caddr (last tt)) 0 1) "_C")
              )
              ( (and (>= (angle3d (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (cadar tt) (caar tt))) (list 0.0 0.0 0.0) (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (caar tt) (car (last tt))))) (* 0.5 pi)) (<= (angle3d (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (caar tt) (cadar tt))) (list 0.0 0.0 0.0) (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (cadar tt) (car (last tt))))) (* 0.5 pi)))
                (vl-cmdf "_.PLINE" "_non" (trans (cadr (last tt)) 0 1) "_non" (trans (caar tt) 0 1) "_non" (trans (cadar tt) 0 1) "_non" (trans (append (polar (mapcar (function +) (list 0.0 0.0) (cadar tt)) (+ (angle (caar tt) (cadar tt)) (* 0.5 pi)) (* 1e+5 (cos (cvunit ang "degree" "radian")))) (list (* 1e+5 (sin (cvunit ang "degree" "radian"))))) 0 1) "_C")
              )
              ( (and (<= (angle3d (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (cadar tt) (caar tt))) (list 0.0 0.0 0.0) (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (caar tt) (car (last tt))))) (* 0.5 pi)) (>= (angle3d (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (caar tt) (cadar tt))) (list 0.0 0.0 0.0) (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (cadar tt) (car (last tt))))) (* 0.5 pi)))
                (vl-cmdf "_.PLINE" "_non" (trans (append (polar (mapcar (function +) (list 0.0 0.0) (caar tt)) (+ (angle (caar tt) (cadar tt)) (* 0.5 pi)) (* 1e+5 (cos (cvunit ang "degree" "radian")))) (list (* 1e+5 (sin (cvunit ang "degree" "radian"))))) 0 1) "_non" (trans (caar tt) 0 1) "_non" (trans (cadar tt) 0 1) "_non" (trans (caddr (last tt)) 0 1) "_C")
              )
              ( (and (<= (angle3d (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (cadar tt) (caar tt))) (list 0.0 0.0 0.0) (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (caar tt) (car (last tt))))) (* 0.5 pi)) (<= (angle3d (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (caar tt) (cadar tt))) (list 0.0 0.0 0.0) (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (cadar tt) (car (last tt))))) (* 0.5 pi)))
                (vl-cmdf "_.PLINE" "_non" (trans (append (polar (mapcar (function +) (list 0.0 0.0) (caar tt)) (+ (angle (caar tt) (cadar tt)) (* 0.5 pi)) (* 1e+5 (cos (cvunit ang "degree" "radian")))) (list (* 1e+5 (sin (cvunit ang "degree" "radian"))))) 0 1) "_non" (trans (caar tt) 0 1) "_non" (trans (cadar tt) 0 1) "_non" (trans (append (polar (mapcar (function +) (list 0.0 0.0) (cadar tt)) (+ (angle (caar tt) (cadar tt)) (* 0.5 pi)) (* 1e+5 (cos (cvunit ang "degree" "radian")))) (list (* 1e+5 (sin (cvunit ang "degree" "radian"))))) 0 1) "_C")
              )
            )
            (cond
              ( (and (>= (angle3d (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (cadar tt) (caar tt))) (list 0.0 0.0 0.0) (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (car (last tt)) (caar tt)))) (* 0.5 pi)) (>= (angle3d (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (caar tt) (cadar tt))) (list 0.0 0.0 0.0) (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (car (last tt)) (cadar tt)))) (* 0.5 pi)))
                (vl-cmdf "_.PLINE" "_non" (trans (car (last tt)) 0 1) "_non" (trans (caar tt) 0 1) "_non" (trans (cadar tt) 0 1) "_C")
              )
              ( (and (>= (angle3d (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (cadar tt) (caar tt))) (list 0.0 0.0 0.0) (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (car (last tt)) (caar tt)))) (* 0.5 pi)) (<= (angle3d (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (caar tt) (cadar tt))) (list 0.0 0.0 0.0) (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (car (last tt)) (cadar tt)))) (* 0.5 pi)))
                (vl-cmdf "_.PLINE" "_non" (trans (car (last tt)) 0 1) "_non" (trans (caar tt) 0 1) "_non" (trans (cadar tt) 0 1) "_non" (trans (append (polar (mapcar (function +) (list 0.0 0.0) (cadar tt)) (+ (angle (caar tt) (cadar tt)) (* 0.5 pi)) (* 1e+5 (cos (cvunit ang "degree" "radian")))) (list (* 1e+5 (sin (cvunit ang "degree" "radian"))))) 0 1) "_C")
              )
              ( (and (<= (angle3d (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (cadar tt) (caar tt))) (list 0.0 0.0 0.0) (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (car (last tt)) (caar tt)))) (* 0.5 pi)) (>= (angle3d (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (caar tt) (cadar tt))) (list 0.0 0.0 0.0) (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (car (last tt)) (cadar tt)))) (* 0.5 pi)))
                (vl-cmdf "_.PLINE" "_non" (trans (append (polar (mapcar (function +) (list 0.0 0.0) (caar tt)) (+ (angle (caar tt) (cadar tt)) (* 0.5 pi)) (* 1e+5 (cos (cvunit ang "degree" "radian")))) (list (* 1e+5 (sin (cvunit ang "degree" "radian"))))) 0 1) "_non" (trans (caar tt) 0 1) "_non" (trans (cadar tt) 0 1) "_non" (trans (car (last tt)) 0 1) "_C")
              )
              ( (and (<= (angle3d (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (cadar tt) (caar tt))) (list 0.0 0.0 0.0) (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (car (last tt)) (caar tt)))) (* 0.5 pi)) (<= (angle3d (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (caar tt) (cadar tt))) (list 0.0 0.0 0.0) (mapcar (function +) (list 0.0 0.0) (mapcar (function -) (car (last tt)) (cadar tt)))) (* 0.5 pi)))
                (vl-cmdf "_.PLINE" "_non" (trans (append (polar (mapcar (function +) (list 0.0 0.0) (caar tt)) (+ (angle (caar tt) (cadar tt)) (* 0.5 pi)) (* 1e+5 (cos (cvunit ang "degree" "radian")))) (list (+ (* 1e+5 (sin (cvunit ang "degree" "radian"))) (cdr (assoc 38 lwx))))) 0 1) "_non" (trans (caar tt) 0 1) "_non" (trans (cadar tt) 0 1) "_non" (trans (append (polar (mapcar (function +) (list 0.0 0.0) (cadar tt)) (+ (angle (caar tt) (cadar tt)) (* 0.5 pi)) (* 1e+5 (cos (cvunit ang "degree" "radian")))) (list (* 1e+5 (sin (cvunit ang "degree" "radian"))))) 0 1) "_C")
              )
            )
          )
          (vl-cmdf "_.PLINE" "_non" (trans (car (last tt)) 0 1) "_non" (trans (cadr (last tt)) 0 1) "_non" (trans (caddr (last tt)) 0 1) "_non" (trans (cadddr (last tt)) 0 1) "_C")
        )
        (setq 3df (entlast))
        (if (not (vl-catch-all-error-p (setq var (vl-catch-all-apply (function vla-addregion) (list (vlax-ename->vla-object (cdr (assoc 330 lwx))) (vlax-safearray-fill (vlax-make-safearray vlax-vbObject '(0 . 0)) (list (vlax-ename->vla-object 3df))))))))
          (setq regobj (car (safearray-value (variant-value var))))
          (setq regobj (vlax-ename->vla-object 3df))
        )
        (if regobj
          (entdel 3df)
        )
        (vl-cmdf "_.UCS" "_P")
        (vl-cmdf "_.EXTRUDE" (vlax-vla-object->ename regobj) "" "_D" "_non" (list 0.0 0.0 0.0) "_non" (list 0.0 0.0 1e+5))
        (ssadd (entlast) ss)
      )
      (if (not (vl-catch-all-error-p (setq var (vl-catch-all-apply (function vla-addregion) (list (vlax-ename->vla-object (cdr (assoc 330 lwx))) (vlax-safearray-fill (vlax-make-safearray vlax-vbObject '(0 . 0)) (list (vlax-ename->vla-object lw))))))))
        (setq regobj (car (safearray-value (variant-value var))))
        (setq regobj (vlax-ename->vla-object lw))
      )
      (vl-cmdf "_.EXTRUDE" (vlax-vla-object->ename regobj) "" "_D" "_non" (list 0.0 0.0 0.0) "_non" (list 0.0 0.0 1e+4))
      (setq el (entlast))
      (vl-cmdf "_.SUBTRACT" el "" ss "")
      (setq 3ds (entlast))
      (if (= (cdr (assoc 0 (entget 3ds))) "3DSOLID")
        (if (= ch "2D")
          (progn
            (vl-cmdf "_.XEDGES" 3ds "")
            (while (setq e (entnext (if (null e) 3ds e)))
              (setq lil (cons e lil))
            )
            (entdel 3ds)
            (foreach li lil
              (entupd (cdr (assoc -1 (entmod (mapcar (function (lambda ( x ) (if (vl-position (car x) (list 10 11)) (list (car x) (cadr x) (caddr x) 0.0) x))) (setq lix (entget li)))))))
              (if (vl-every (function (lambda ( x ) (vl-some (function (lambda ( y ) (equal x (append y (list 0.0)) 1e-6))) pl))) (mapcar (function cdr) (vl-remove-if-not (function (lambda ( x ) (vl-position (car x) (list 10 11)))) lix)))
                (entdel li)
              )
            )
          )
        )
      )
      (prompt "\nElapsed time : ") (princ (rtos (- (car (_vl-times)) ti) 2 16)) (prompt " milliseconds...")
    )
  )
  (if ucsf
    (vl-cmdf "_.UCS" "_P")
  )
  (vl-cmdf "_.UNDO" "_E")
  (*error* nil)
)